from .tokens import Tokens
import json


class Parser(object):
    """Parser input like - container="$container" path="$path" source="$source"
     into [$container, $path, $source]"""
    @classmethod
    def parse(self, line: str):
        result = {}
        # splits by spaces
        options = line.split()
        for option in options:
            if option.__contains__(Tokens.ContainerToken):
                result[Tokens.Container] = self.normalize(option.replace(
                    Tokens.ContainerToken, ""))
            elif option.__contains__(Tokens.PathToken):
                result[Tokens.Path] = self.normalize(option.replace(
                    Tokens.PathToken, ""))

            elif option.__contains__(Tokens.ProviderToken):
                result[Tokens.Provider] = self.normalize(option.replace(
                    Tokens.ProviderToken, ""))

            elif option.__contains__(Tokens.ProviderOptionsToken):
                result[Tokens.ProviderOptions] = json.loads(option.replace(
                    Tokens.ProviderOptionsToken, ""))
            elif option.__contains__(Tokens.NavigateToken):
                result[Tokens.Navigate] = json.loads(option.replace(
                    Tokens.NavigateToken, ""))
        return result

    @classmethod
    def normalize(self, data: str):
        data = data.replace('"', '')
        data = data.replace("'", '')
        return data



from IPython.core.magic import magics_class, line_magic, Magics, Configurable
from IPython.display import display, Javascript
from IPython.lib import backgroundjobs as bg

from .constants import Constants
from .tokens import Tokens
from .parser import Parser
from ..providers.provider import Provider
from ..providers.constants import Constants as ProviderConstants
from ..providers.github import GithubProvider
from .notebook import write, NoteBookUrl, ExtensionName

import logging
import asyncio
import time
import websocket
import json
from threading import Thread
import random
import ssl


_MAGIC_NAME = "nimport"
_WS_BaseUrl = "wss://connect.websocket.in/nimport?room_id={0}"
_Result_Content = "content"
_Result_Url = "url"
jobs = bg.BackgroundJobManager()


@magics_class
class Nimportmagic(Magics, Configurable):
    _providers = {}

    githubProvider = GithubProvider()
    _providers[ProviderConstants.GitHub] = githubProvider

    _gotMessage: bool = False

    _result = {}

    @line_magic(Constants.MAGIC_NAME)
    def run(self, line):
        display(line)
        results = Parser.parse(line)
        providerName = results[Tokens.Provider]
        provider = self._providers[providerName]
        if provider:
            providerResult = provider.get(
                results[Tokens.Container], results[Tokens.Path], results[Tokens.ProviderOptions])
            self._result[_Result_Content] = providerResult
            self.waitForParams(results, provider)
        else:
            display("Unknown provider: " + providerName)

    @classmethod
    def waitForParams(cls, inputs, provider):
        display("Trying to get the current link...")
        url = _WS_BaseUrl.format(cls.getRoomId())
        ws = websocket.create_connection(
            url, sslopt={"cert_reqs": ssl.CERT_NONE})
        display(Javascript(cls.getJSWsContent(url)))
        data = json.loads(ws.recv())
        cls._result[_Result_Url] = data[_Result_Url]
        ws.close()
        cls.postContent(inputs, provider)

    @classmethod
    def postContent(cls, inputs, provider):
        display("Got link...making final adjustments")
        content = cls._result[_Result_Content]
        url = cls._result[_Result_Url]
        newNoteBookName = inputs[Tokens.Path]
        if not content:
            # this is clone so notebook name should have the repo name
            newNoteBookName = provider.getContainerName(
                inputs[Tokens.Container]) + "/" + newNoteBookName

        notebookUrl = NoteBookUrl(url)
        if content:
            write(content, newNoteBookName)
        newUrl = notebookUrl.getNewLink(newNoteBookName)
        if Tokens.Navigate in inputs:
            display(Javascript(cls.getJSPostContent(newUrl)))
        else:
            display("Navigate to:")
            display(newUrl)

    @classmethod
    def getJSWsContent(cls, url):
        js = """
        var websocket = new WebSocket("%s");

        websocket.onopen = function (event) {
            websocket.send(JSON.stringify({
                url: window.location.href
            }));
        };
        """ % (url)
        return js

    @classmethod
    def getJSPostContent(cls, url):
        js = """
        if (window) {
            window.location.href = "%s"
        }
        """ % (url)
        return js

    @classmethod
    def getRoomId(cls):
        return random.randint(0, 99999999999999999999)



'''
https://ipython.readthedocs.io/en/stable/config/extensions/
'''
from .lib.nimport_magic import Nimportmagic as Magic

def load_ipython_extension(ip):
    ip.register_magics(Magic)


def unload_ipython_extension(ip):
    # nothing here for now
    print("unloaded")



from setuptools import setup, find_packages

NAME = "nimport"
VERSION = '0.10.7'

INSTALL_REQUIRES = [
    'PyGithub',
    'websocket-client',
    'IPython',
    'nbformat',
]

setup(
    name=NAME,
    version=VERSION,
    install_requires=INSTALL_REQUIRES,
    packages=find_packages(exclude=['tests'])
)



class Tokens(object):
    Container = "container"
    Path = "path"
    Provider = "provider"
    ProviderOptions = "providerOptions"
    Navigate = "navigate"

    Equals = "="

    ContainerToken = Container + Equals
    PathToken = Path + Equals
    ProviderToken = Provider + Equals
    ProviderOptionsToken = ProviderOptions + Equals
    NavigateToken = Navigate + Equals
